import argparse
import logging
import sys

from datacatalog_fileset_enricher import datacatalog_fileset_enricher
from datacatalog_fileset_exporter import datacatalog_fileset_exporter_cli
from datacatalog_fileset_processor import datacatalog_fileset_processor_cli
from datacatalog_tag_exporter import datacatalog_tag_exporter_cli
from datacatalog_tag_manager import tag_datasource_processor
from datacatalog_tag_template_exporter import datacatalog_tag_template_exporter_cli
from datacatalog_tag_template_processor import datacatalog_tag_template_processor_cli
from datacatalog_object_storage_processor import datacatalog_object_storage_processor_cli


class DatacatalogUtilsCLI:

    @classmethod
    def run(cls, argv):
        cls.__setup_logging()

        args = cls._parse_args(argv)
        args.func(args)

    @classmethod
    def __setup_logging(cls):
        logging.basicConfig(level=logging.INFO)

    @classmethod
    def _parse_args(cls, argv):
        parser = argparse.ArgumentParser(description=__doc__,
                                         formatter_class=argparse.RawDescriptionHelpFormatter)

        subparsers = parser.add_subparsers()

        cls.add_tag_templates_cmd(subparsers)

        cls.add_tags_cmd(subparsers)

        cls.add_fileset_enricher_cmd(subparsers)

        datacatalog_object_storage_processor_cli.DatacatalogObjectStorageProcessorCLI.\
            add_object_storage_parser(subparsers)

        return parser.parse_args(argv)

    @classmethod
    def add_tags_cmd(cls, subparsers):
        tags_parser = subparsers.add_parser("tags", help="Tags commands")

        tags_subparsers = tags_parser.add_subparsers()

        cls.add_create_tags_cmd(tags_subparsers)

        cls.add_delete_tags_cmd(tags_subparsers)

        datacatalog_tag_exporter_cli.DatacatalogTagExporterCLI.add_export_tags_cmd(tags_subparsers)

    @classmethod
    def add_tag_templates_cmd(cls, subparsers):
        tag_templates_parser = subparsers.add_parser("tag-templates",
                                                     help="Tag Templates commands")

        tag_templates_subparsers = tag_templates_parser.add_subparsers()

        datacatalog_tag_template_processor_cli.DatacatalogTagTemplateProcessorCLI.\
            add_create_tag_templates_cmd(tag_templates_subparsers)

        datacatalog_tag_template_processor_cli.DatacatalogTagTemplateProcessorCLI.\
            add_delete_tag_templates_cmd(tag_templates_subparsers)

        datacatalog_tag_template_exporter_cli.DatacatalogTagTemplateExporterCLI.\
            add_export_tag_templates_cmd(tag_templates_subparsers)

    @classmethod
    def add_fileset_enricher_cmd(cls, subparsers):
        filesets_parser = subparsers.add_parser("filesets", help="Filesets commands")

        filesets_subparsers = filesets_parser.add_subparsers()

        datacatalog_fileset_processor_cli.DatacatalogFilesetProcessorCLI.\
            add_create_filesets_cmd(filesets_subparsers)

        enrich_filesets = filesets_subparsers.add_parser('enrich',
                                                         help='Enrich GCS filesets with Tags')

        enrich_filesets.add_argument('--project-id', help='Project id', required=True)
        enrich_filesets.add_argument('--tag-template-name',
                                     help='Name of the Fileset Enrich template,'
                                     'i.e: '
                                     'projects/my-project/locations/us-central1/tagTemplates/'
                                     'my_template_test')

        enrich_filesets.add_argument('--entry-group-id', help='Entry Group ID')
        enrich_filesets.add_argument('--entry-id', help='Entry ID')
        enrich_filesets.add_argument('--tag-fields',
                                     help='Specify the fields you want on the generated Tags,'
                                     ' split by comma, use the list available in the docs')
        enrich_filesets.add_argument('--bucket-prefix',
                                     help='Specify a bucket prefix if you want to avoid scanning'
                                     ' too many GCS buckets')
        enrich_filesets.set_defaults(func=cls.__enrich_fileset)

        clean_up_tags = filesets_subparsers.add_parser(
            'clean-up-templates-and-tags',
            help='Clean up the Fileset Enhancer Template and Tags From the Fileset Entries')
        clean_up_tags.add_argument('--project-id', help='Project id', required=True)

        clean_up_tags.set_defaults(func=cls.__clean_up_fileset_template_and_tags)

        datacatalog_fileset_processor_cli.DatacatalogFilesetProcessorCLI.\
            add_delete_filesets_cmd(filesets_subparsers)

        # ADD filesets export command.
        datacatalog_fileset_exporter_cli.DatacatalogFilesetExporterCLI.add_export_filesets_cmd(
            filesets_subparsers)

    @classmethod
    def add_create_tags_cmd(cls, subparsers):
        create_tags_parser = subparsers.add_parser('create', help='Create Tags from CSV')
        create_tags_parser.add_argument('--csv-file',
                                        help='CSV file with Tags information',
                                        required=True)
        create_tags_parser.set_defaults(func=cls.__create_tags)

    @classmethod
    def add_delete_tags_cmd(cls, subparsers):
        create_tags_parser = subparsers.add_parser('delete', help='Delete Tags from CSV')
        create_tags_parser.add_argument('--csv-file',
                                        help='CSV file with Tags information',
                                        required=True)
        create_tags_parser.set_defaults(func=cls.__delete_tags)

    @classmethod
    def __enrich_fileset(cls, args):
        tag_fields = None
        if args.tag_fields:
            tag_fields = args.tag_fields.split(',')

        datacatalog_fileset_enricher.DatacatalogFilesetEnricher(args.project_id).run(
            args.entry_group_id, args.entry_id, tag_fields, args.bucket_prefix,
            args.tag_template_name)

    @classmethod
    def __clean_up_fileset_template_and_tags(cls, args):
        datacatalog_fileset_enricher.DatacatalogFilesetEnricher(
            args.project_id).clean_up_fileset_template_and_tags()

    @classmethod
    def __create_tags(cls, args):
        tag_datasource_processor.TagDatasourceProcessor().create_tags_from_csv(
            file_path=args.csv_file)

    @classmethod
    def __delete_tags(cls, args):
        tag_datasource_processor.TagDatasourceProcessor().delete_tags_from_csv(
            file_path=args.csv_file)


def main():
    argv = sys.argv
    DatacatalogUtilsCLI.run(argv[1:] if len(argv) > 0 else argv)


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



"""Top-level package for datacatalog_util."""

from .datacatalog_util_cli import main  # noqa: F401

__author__ = """Marcelo Miranda"""
__email__ = 'mesmacosta@gmail.com'
__version__ = '0.11.1'

__all__ = ('main')



import unittest
from unittest import mock

import datacatalog_util
from datacatalog_util import datacatalog_util_cli


class TagManagerCLITest(unittest.TestCase):

    def test_parse_args_invalid_subcommand_should_raise_system_exit(self):
        self.assertRaises(SystemExit, datacatalog_util_cli.DatacatalogUtilsCLI._parse_args,
                          ['invalid-subcommand'])

    def test_parse_args_create_tags_missing_mandatory_args_should_raise_system_exit(self):
        self.assertRaises(SystemExit, datacatalog_util_cli.DatacatalogUtilsCLI._parse_args,
                          ['tags', 'create'])

    def test_parse_args_create_tags_should_parse_mandatory_args(self):
        args = datacatalog_util_cli.DatacatalogUtilsCLI._parse_args(
            ['tags', 'create', '--csv-file', 'test.csv'])
        self.assertEqual('test.csv', args.csv_file)

    def test_run_no_args_should_raise_attribute_error(self):
        self.assertRaises(AttributeError, datacatalog_util_cli.DatacatalogUtilsCLI.run, None)

    @mock.patch(
        'datacatalog_util.datacatalog_util_cli.tag_datasource_processor.TagDatasourceProcessor')
    def test_run_create_tags_should_call_tag_creator(self, mock_tag_datasource_processor):
        datacatalog_util_cli.DatacatalogUtilsCLI.run(['tags', 'create', '--csv-file', 'test.csv'])

        tag_datasource_processor = mock_tag_datasource_processor.return_value
        tag_datasource_processor.create_tags_from_csv.assert_called_once()
        tag_datasource_processor.create_tags_from_csv.assert_called_with(file_path='test.csv')

    @mock.patch(
        'datacatalog_util.datacatalog_util_cli.tag_datasource_processor.TagDatasourceProcessor')
    def test_run_delete_tags_should_call_tag_creator(self, mock_tag_datasource_processor):
        datacatalog_util_cli.DatacatalogUtilsCLI.run(['tags', 'delete', '--csv-file', 'test.csv'])

        tag_datasource_processor = mock_tag_datasource_processor.return_value
        tag_datasource_processor.delete_tags_from_csv.assert_called_once()
        tag_datasource_processor.delete_tags_from_csv.assert_called_with(file_path='test.csv')

    @mock.patch('datacatalog_tag_template_processor.tag_template_datasource_processor.'
                'TagTemplateDatasourceProcessor')
    def test_run_create_tag_templates_should_call_tag_template_creator(
            self, mock_tag_template_datasource_processor):  # noqa: E125

        datacatalog_util_cli.DatacatalogUtilsCLI.run(
            ['tag-templates', 'create', '--csv-file', 'test.csv'])

        tag_template_datasource_processor = mock_tag_template_datasource_processor.return_value
        tag_template_datasource_processor.create_tag_templates_from_csv.assert_called_once()
        tag_template_datasource_processor.create_tag_templates_from_csv.assert_called_with(
            file_path='test.csv')

    @mock.patch('datacatalog_tag_template_processor.tag_template_datasource_processor.'
                'TagTemplateDatasourceProcessor')
    def test_run_delete_tag_templates_should_call_correct_method(
            self, mock_tag_template_datasource_processor):  # noqa: E125

        datacatalog_util_cli.DatacatalogUtilsCLI.run(
            ['tag-templates', 'delete', '--csv-file', 'test.csv'])

        tag_template_datasource_processor = mock_tag_template_datasource_processor.return_value
        tag_template_datasource_processor.delete_tag_templates_from_csv.assert_called_once()
        tag_template_datasource_processor.delete_tag_templates_from_csv.assert_called_with(
            file_path='test.csv')

    @mock.patch('datacatalog_tag_template_exporter.tag_template_datasource_exporter.'
                'TagTemplateDatasourceExporter')
    def test_run_export_tag_templates_should_call_correct_method(
            self, mock_tag_template_datasource_exporter):  # noqa: E125

        datacatalog_util_cli.DatacatalogUtilsCLI.run([
            'tag-templates', 'export', '--file-path', 'test.csv', '--project-ids',
            'my-project1,my-project2'
        ])

        tag_template_datasource_processor = mock_tag_template_datasource_exporter.return_value
        tag_template_datasource_processor.export_tag_templates.assert_called_once()
        tag_template_datasource_processor.export_tag_templates.assert_called_with(
            project_ids='my-project1,my-project2', file_path='test.csv')

    @mock.patch('datacatalog_tag_exporter.tag_datasource_exporter.TagDatasourceExporter')
    def test_run_export_tags_should_call_correct_method(self, mock_tag_datasource_exporter):
        datacatalog_util_cli.DatacatalogUtilsCLI.run([
            'tags', 'export', '--dir-path', 'test.csv', '--project-ids', 'my-project1,my-project2'
        ])

        tag_datasource_processor = mock_tag_datasource_exporter.return_value
        tag_datasource_processor.export_tags.assert_called_once()
        tag_datasource_processor.export_tags.assert_called_with(
            date_created=None,
            project_ids='my-project1,my-project2',
            dir_path='test.csv',
            tag_templates_names=None)

    @mock.patch('datacatalog_util.datacatalog_util_cli.'
                'datacatalog_fileset_enricher.DatacatalogFilesetEnricher')
    def test_run_filesets_enrich_should_call_correct_method(self, mock_fileset_enricher):
        datacatalog_util_cli.DatacatalogUtilsCLI.run(
            ['filesets', 'enrich', '--project-id', 'my-project1'])

        mock_fileset_enricher_processor = mock_fileset_enricher.return_value
        mock_fileset_enricher_processor.run.assert_called_once()
        mock_fileset_enricher_processor.run.assert_called_with(None, None, None, None, None)

    @mock.patch('datacatalog_util.datacatalog_util_cli.'
                'datacatalog_fileset_enricher.DatacatalogFilesetEnricher')
    def test_run_filesets_cleanup_should_call_correct_method(self, mock_fileset_enricher):
        datacatalog_util_cli.DatacatalogUtilsCLI.run(
            ['filesets', 'clean-up-templates-and-tags', '--project-id', 'my-project1'])

        mock_fileset_enricher_processor = mock_fileset_enricher.return_value
        mock_fileset_enricher_processor.clean_up_fileset_template_and_tags.assert_called_once()

    @mock.patch('datacatalog_fileset_processor.'
                'fileset_datasource_processor.FilesetDatasourceProcessor')
    def test_run_filesets_create_should_call_correct_method(self, mock_fileset_create):
        datacatalog_util_cli.DatacatalogUtilsCLI.run(
            ['filesets', 'create', '--csv-file', 'test.csv'])

        mock_fileset_datasource_processor = mock_fileset_create.return_value
        mock_fileset_datasource_processor.create_entry_groups_and_entries_from_csv.\
            assert_called_once()
        mock_fileset_datasource_processor.create_entry_groups_and_entries_from_csv(
            file_path='test.csv')

    @mock.patch('datacatalog_fileset_processor.'
                'fileset_datasource_processor.FilesetDatasourceProcessor')
    def test_run_filesets_delete_should_call_correct_method(self, mock_fileset_delete):
        datacatalog_util_cli.DatacatalogUtilsCLI.run(
            ['filesets', 'delete', '--csv-file', 'test.csv'])

        mock_fileset_datasource_processor = mock_fileset_delete.return_value
        mock_fileset_datasource_processor.delete_entry_groups_and_entries_from_csv.\
            assert_called_once()
        mock_fileset_datasource_processor.delete_entry_groups_and_entries_from_csv(
            file_path='test.csv')

    @mock.patch('datacatalog_fileset_exporter.fileset_datasource_exporter.'
                'FilesetDatasourceExporter')
    def test_run_export_filesets_should_call_correct_method(self, mock_fileset_exporter):
        datacatalog_util_cli.DatacatalogUtilsCLI.run([
            'filesets', 'export', '--file-path', 'test.csv', '--project-ids',
            'my-project1,my-project2'
        ])

        filset_datasource_exporter = mock_fileset_exporter.return_value
        filset_datasource_exporter.export_filesets.assert_called_once()
        filset_datasource_exporter.export_filesets.assert_called_with(
            date_created=None, project_ids='my-project1,my-project2', file_path='test.csv')

    @mock.patch('datacatalog_util.datacatalog_util_cli.DatacatalogUtilsCLI')
    def test_main_should_call_cli_run(self, mock_cli):
        datacatalog_util.main()
        mock_cli.run.assert_called_once()
